--[[
	GemCensus: A gem popularity addon.
	Copyright (c) 2010 Cyprias <Cyprias@gmail.com>
	All Rights Reserved.
	Only Curse.com, WoWInterface.com & WoWace.com have permission to redistribute this software. 
	GemCensus shall not be included in a UI pack / compilation without permission. 
]]

local folder = "GemCensus_Display"
local core = LibStub("AceAddon-3.0"):GetAddon("GemCensus")
local display = core.display
local L = core.L

local iLink = core.iLink
local UIParent = UIParent
local TradeSkillFrameAvailableFilterCheckButton = TradeSkillFrameAvailableFilterCheckButton
local select = select
local GetTradeSkillInvSlots = GetTradeSkillInvSlots
local GetTradeSkillInvSlotFilter = GetTradeSkillInvSlotFilter
local GetTradeSkillSubClasses = GetTradeSkillSubClasses
local GetTradeSkillSubClassFilter = GetTradeSkillSubClassFilter
local TradeSkillOnlyShowMakeable = TradeSkillOnlyShowMakeable
local SetTradeSkillSubClassFilter = SetTradeSkillSubClassFilter
local SetTradeSkillInvSlotFilter = SetTradeSkillInvSlotFilter
local GetFirstTradeSkill = GetFirstTradeSkill
local GetNumTradeSkills = GetNumTradeSkills
local GetTradeSkillInfo = GetTradeSkillInfo
local ExpandTradeSkillSubClass = ExpandTradeSkillSubClass
local GetTradeSkillItemLink = GetTradeSkillItemLink
local DoTradeSkill = DoTradeSkill
local GetNumFactions = GetNumFactions
local CollapseTradeSkillSubClass = CollapseTradeSkillSubClass

local GetTradeSkillLine = GetTradeSkillLine
local echo = core.echo
local iID = core.iID
local Debug = core.Debug


--locale name
local Jewelcrafting = GetSpellInfo(25229) --Jewelcrafting

local LPT = LibStub("LibPeriodicTable-3.1")
if not LPT then
	error(folder .. " requires LibPeriodicTable-3.1") 
	return
end

display.LPT = LPT

local GetItemInfo = GetItemInfo
local pairs = pairs

local rawGemNames = {}
display.cutToRaw = setmetatable({}, {
	__index = function(t,cutID)
	local cutName = GetItemInfo(cutID)
	for rawID, rawName in pairs(rawGemNames) do 
		if cutName:find(rawName) then
			t[cutID] = rawID
			return t[cutID]
		end
	end

	t[cutID] = false
	return t[cutID]
	end
})

function display:GetRawGems()--		/run _GCGem:GetRawGems()
	local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount,itemEquipLoc, itemTexture, itemSellPrice
	for item, value, set in LPT:IterateSet("Tradeskill.Mat.ByProfession.Jewelcrafting") do
		itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount,itemEquipLoc, itemTexture, itemSellPrice = GetItemInfo(item)
		if itemType == L.gem then
			rawGemNames[item] = itemName
		end
	end
end

local tooltip = CreateFrame("GameTooltip", folder.."ToolTip", UIParent, "GameTooltipTemplate")
local tooltipStats = _G[tooltip:GetName().."TextLeft2"]
tooltip:SetOwner(UIParent, "ANCHOR_NONE")

--Cache gem stats to a table. eg "+23 Spell Power"
display.gemStats = setmetatable({}, {
	__index = function(t,itemID)
		t[itemID] = "*"
		
		local gemLink = itemID and iLink[itemID]
		if gemLink then
			tooltip:SetOwner(UIParent, "ANCHOR_NONE")
			tooltip:ClearLines()
			tooltip:SetHyperlink(gemLink)
			
			local text = tooltipStats:GetText()
			if text and text:trim() ~= "" then
				t[itemID] = tooltipStats:GetText()
			end
		end
		return t[itemID]
	end
})

function display:CraftItem(itemID, times)
--~ 	local skillName, skillType, numAvailable, isExpanded, itemLink
	
	if GetTradeSkillLine() ~= Jewelcrafting then
		echo(L.openJCwindow)
		return
	end
	
	local times = times or 1
	
	local collapsedHeaders, showMakeable, subClass, invSlot, search = core:ResetTradeskillFrame()

	local cID, skillName, skillType, numAvailable, isExpanded, itemLink
	for i = GetFirstTradeSkill(), GetNumTradeSkills()  do --count
		skillName, skillType, numAvailable, isExpanded = GetTradeSkillInfo(i);
		if ( skillType ~= "header" ) then
			itemLink = GetTradeSkillItemLink(i)
			if itemLink then
				cID = iID[itemLink]
				if cID == itemID then
--~ 					Debug("CraftItem", "tID:"..i, itemLink.."x"..times)
					
					echo(L.craftingGems:format(itemLink, times))
					DoTradeSkill(i, times)
					break
				end
			end
		end
	end

	core:RestoreTradeskillFrame(collapsedHeaders, showMakeable, subClass, invSlot, search)
end


